﻿using System;
using System.Collections.Generic;
using System.IO;
using System.ServiceModel.Channels;
using System.Threading;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus.Messaging;

namespace MSDNReceiver
{
    class Program
    {
        //Windows Azure Service Bus properties
        static string serviceBusNamespace = "msdnservicebus";
        static string serviceBusIssuerName = "owner";
        static string serviceBusIssuerKey = "B5cNsviiKqrGfQcuUD4jHcVLM6je4aCC31PdrMVBOkI=";
        static string topicName = "headquarterstopic";
        
        static int receiverDelay = 200;

        static ConsoleColor[] colors = new ConsoleColor[] { 
            ConsoleColor.Red, 
            ConsoleColor.Green, 
            ConsoleColor.Yellow, 
            ConsoleColor.Cyan,
            ConsoleColor.Magenta,
            ConsoleColor.Blue,             
            ConsoleColor.White};

        static void Main(string[] args)
        {
            Console.WriteLine("Enter Store Number");
            string storeNumber = Console.ReadLine();

            Console.WriteLine("Selecting Subscription for Store...");

            //Create a Subscription Client to the Topic
            Uri runtimeUri = ServiceBusEnvironment.CreateServiceUri("sb", serviceBusNamespace, string.Empty);
            TokenProvider tokenProvider = TokenProvider.CreateSharedSecretTokenProvider(serviceBusIssuerName, serviceBusIssuerKey);
            MessagingFactory messagingFactory = MessagingFactory.Create(runtimeUri, tokenProvider);
            NamespaceManager namespaceManager = new NamespaceManager(runtimeUri, TokenProvider.CreateSharedSecretTokenProvider(serviceBusIssuerName, serviceBusIssuerKey));
            SubscriptionClient subscriptionClient = messagingFactory.CreateSubscriptionClient(topicName, "Store" + storeNumber.Trim() + "Sub", ReceiveMode.PeekLock);

            Console.Clear();
            Console.WriteLine("Ready to receive messages from {0}...", subscriptionClient.Name);

            bool allSessionsAccepted = false;
            while (!allSessionsAccepted)
            {
                try
                {
                    Console.WriteLine("Checking for session...");
                    MessageSession sessionReceiver = subscriptionClient.AcceptMessageSession(TimeSpan.FromSeconds(5));

                    ReceiveSessionMessages(sessionReceiver);
                    Console.WriteLine("All received on this session");
                }
                catch (TimeoutException)
                {
                    Console.WriteLine("Got TimeoutException, no more sessions available");
                    allSessionsAccepted = true;
                }
            }

            Console.WriteLine("\nReceiver complete.");
            Console.ReadLine();

        }

        static void ReceiveSessionMessages(MessageSession receiver)
        {
            // Read messages from subscription until subscription is empty
            Console.WriteLine("Reading messages from subscription {0}", receiver.Path);
            Console.WriteLine("Receiver Type:" + receiver.GetType().Name);
            Console.WriteLine("Receiver.SessionId = " + receiver.SessionId);

            SequenceState sessionState = GetState(receiver);
            BrokeredMessage receivedMessage;
            while ((receivedMessage = receiver.Receive()) != null)
            {
                string sessionId = receiver.SessionId;
                ProcessMessage(receivedMessage, ref sessionState, receiver);
                while (sessionState.GetNextOutOfSequenceMessage() != -1)
                {
                    //Call back deferred messages
                    Console.WriteLine("Calling back for deferred message: Category {0}, Message sequence {1}", receiver.SessionId, sessionState.GetNextSequenceId());
                    receivedMessage = receiver.Receive(sessionState.GetNextOutOfSequenceMessage());
                    ProcessMessage(receivedMessage, ref sessionState, receiver);
                }

                if (receivedMessage.Properties.ContainsKey("IsLastMessageInSession"))
                    break;

            }
            
            SetState(receiver, null);
            receiver.Close();
        }

        static void ProcessMessage(BrokeredMessage message, ref SequenceState sessionState, MessageSession session = null)
        {
            if (session != null)
            {
                int messageId = Convert.ToInt32(message.Properties["CatalogPage"]);
                if (sessionState.GetNextSequenceId() == messageId)
                {
                    OutputMessageInfo("RECV: ", message, "State: " + "RECEIVED");
                    sessionState.SetNextSequenceId(messageId + 1);
                    message.Complete();
                    SetState(session, sessionState);
                }
                else
                {
                    Console.WriteLine("Deferring message: Category {0}, Catalog Page {1}", session.SessionId, messageId);
                    sessionState.AddOutOfSequenceMessage(messageId, message.SequenceNumber);
                    message.Defer();
                    SetState(session, sessionState);
                }
            }

            Thread.Sleep(receiverDelay);
        }

        static SequenceState GetState(MessageSession session)
        {
            SequenceState state = new SequenceState();
            Stream stream = session.GetState();

            if (stream != null)
            {
                using (stream)
                {
                    System.Xml.Serialization.XmlSerializer reader = new System.Xml.Serialization.XmlSerializer(typeof(SequenceState));
                    state = (SequenceState)reader.Deserialize(stream);
                }
            }

            //Added in case the session state is corrupted
            if (state == null)
                state = new SequenceState();

            return state;
        }

        static void SetState(MessageSession session, SequenceState state)
        {
            using (var stream = new MemoryStream())
            {
                System.Xml.Serialization.XmlSerializer XMLwriter = new System.Xml.Serialization.XmlSerializer(typeof(SequenceState));
                XMLwriter.Serialize(stream, state);
                stream.Position = 0;
                session.SetState(stream);
            }
        }

        public static void OutputMessageInfo(string action, BrokeredMessage message, string additionalText = "")
        {
            if (message == null)
            {
                return;
            }
            int sequenceId = Convert.ToInt32(message.Properties["CatalogPage"]);
            object prop = message.Properties["Category"];
            if (prop != null)
            {
                Console.ForegroundColor = colors[int.Parse(prop.ToString()) % colors.Length];
                Console.WriteLine("{0}{1} - {2}", action, message.MessageId, additionalText);
                Console.ResetColor();
            }
        }
    }
}
