﻿using System;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Description;
using Microsoft.ServiceBus.Messaging;
using System.Threading;
using System.IO;
using System.Xml.Serialization;

namespace MSDNSender
{
    class Program
    {
        //Windows Azure Service Bus properties
        static string serviceBusNamespace = "msdnservicebus";
        static string serviceBusIssuerName = "owner";
        static string serviceBusIssuerKey = "B5cNsviiKqrGfQcuUD4jHcVLM6je4aCC31PdrMVBOkI=";
        static string topicName = "headquarterstopic";
        static int senderDelay = 500;

        static ConsoleColor[] colors = new ConsoleColor[] { 
            ConsoleColor.Red, 
            ConsoleColor.Green, 
            ConsoleColor.Yellow, 
            ConsoleColor.Cyan,
            ConsoleColor.Magenta,
            ConsoleColor.Blue,             
            ConsoleColor.White};
        
        static void Main(string[] args)
        {
            Console.Title = "MessageSender";

            //Create a Topic client using the Service Bus credentials
            Uri runtimeUri = ServiceBusEnvironment.CreateServiceUri("sb", serviceBusNamespace, string.Empty);
            TokenProvider tokenProvider = TokenProvider.CreateSharedSecretTokenProvider(serviceBusIssuerName, serviceBusIssuerKey);
            MessagingFactory messagingFactory = MessagingFactory.Create(runtimeUri, tokenProvider);
            TopicClient topicClient = messagingFactory.CreateTopicClient(topicName);

            Console.WriteLine("Preparing to send messages to {0}...", topicClient.Path);
            Console.WriteLine("Sending messages to topic {0}", topicClient.Path);

            //Deserialize XML file with Products, and store them in an object array
            Catalog catalog = null;
            string path = "ProductsCatalog.xml";
            XmlSerializer serializer = new XmlSerializer(typeof(Catalog));
            StreamReader reader = new StreamReader(path);
            catalog = (Catalog) serializer.Deserialize(reader);
            reader.Close();

            //Loop through the array, and send each product in single messages
            int numberOfElementsInArray = catalog.products.Length;
            int currentArrayElement = 0;
            foreach (Product product in catalog.products)
            {
                Boolean isLastProductInArray = false;
                currentArrayElement++;
                if (currentArrayElement == numberOfElementsInArray)
                    isLastProductInArray = true;

                BrokeredMessage message = CreateMessage(catalog.Name, product, isLastProductInArray);
                try
                {
                    topicClient.Send(message);
                }
                catch (Exception)
                {
                    break;
                }

                OutputMessageInfo("SEND: ", message);
                Thread.Sleep(senderDelay);
            }

            Console.WriteLine("\nPress [Enter] to exit.");
            Console.ReadLine();
        }

        
        static BrokeredMessage CreateMessage(string catalogName, Product product, Boolean isLastProductInArray)
        {
            //Construct message based on properties extracted from the product object
            BrokeredMessage message = new BrokeredMessage();
            message.Properties.Add("Catalog", catalogName);
            message.Properties.Add("ProductId", product.ProductId);
            message.Properties.Add("ProductName", product.ProductName);
            message.Properties.Add("Category", product.Category);
            message.Properties.Add("CatalogPage", product.CatalogPage);
            message.Properties.Add("Store", product.Store);
            if (isLastProductInArray)
                message.Properties.Add("IsLastMessageInSession", "true");
            message.SessionId = catalogName;
            message.MessageId = "Catalog_" + catalogName + "Store_" + product.Store + "ProductName_" + product.ProductName + "Category_" + product.Category + "_CatalogPage_" + product.CatalogPage + "_" + DateTime.Now.ToShortTimeString();
            return message;
        }

        public static void OutputMessageInfo(string action, BrokeredMessage message, string additionalText = "")
        {
            if (message == null)
            {
                return;
            }
            int sequenceId = Convert.ToInt32(message.Properties["CatalogPage"]);
            object prop = message.Properties["Category"];
            if (prop != null)
            {
                Console.ForegroundColor = colors[int.Parse(prop.ToString()) % colors.Length];
                Console.WriteLine("{0}{1} - {2}", action, message.MessageId, additionalText);
                Console.ResetColor();
            }
        }
    }
    
}
